package yrpg.slm.pc.plugin.form;

import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.form.control.Control;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.sdk.plugin.Plugin;
import yrpg.slm.pc.common.BillPushCommon;

import java.util.ArrayList;
import java.util.EventObject;
import java.util.Iterator;
import java.util.List;

/**
 * 动态表单插件
 * 生成发放单  动态表单实现
 */
public class WagesGrantAchieveFormPlugin extends AbstractFormPlugin implements Plugin {

    private final String grantBill = "yrpg_wages_grant";//工资发放单
    private final String balanceBill = "yrpg_pc_awardbalance";//奖项余额表
    private final String awardBill = "yrpg_pc_awardtype";//奖项

    private DataSet awardTypeDataSet = null;//所有奖项
    private DataSet allAwardDataSet = null;//余额表奖项数据
    private DataSet grantAwardDataSet = null;//发放单数据

    /**
     * 监听按钮
     * @param e
     */
    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        // 侦听按钮点击事件
        this.addClickListeners("btnok");

    }

    /**
     * 点击发放事件  生成发放单
     * @param evt
     */
    @Override
    public void click(EventObject evt) {
        super.click(evt);
        Control source = (Control) evt.getSource();
        String name = source.getKey();
        if (name.equals("btnok")) {

            this.getDataSet("begin");//获取所有余额表数据源

            DataSet personDataSet = allAwardDataSet.copy().groupBy(new String []{"yrpg_user"}).finish();
            DynamicObjectCollection personCollection = ORM.create().toPlainDynamicObjectCollection(personDataSet.copy());
            if(personCollection.size() == 0){
                this.getView().showErrMessage("余额表中本期不存在数据","工资发放");
                return;
            }

            List<DynamicObject> addList = new ArrayList<>();
            for(DynamicObject person : personCollection){
                long user = person.getLong("yrpg_user");
                DataSet personAwardDataSet = allAwardDataSet.copy().filter("yrpg_user = "+ user);
                //构建工资发放单
                DynamicObject data = this.createGrantBill(user,personAwardDataSet);
                //添加工资发放单
                addList.add(data);
            }

            //保存  之后采用批量保存
            OperationResult operationResult = SaveServiceHelper.saveOperate("yrpg_wages_grant", addList.toArray(new DynamicObject[] {}),
                    OperateOption.create());
            if(!operationResult.isSuccess()){

            }
            else {
                //所有成功的发放单 遍历所有奖项下推
                this.getDataSet("end");//加载发放单
                this.createAwardBill();
                this.getView().showMessage("工资发放单生成完成");
            }

        }
    }


    /**
     * 构建工资发放单
     * @param personAwardDataSet
     * @return
     */
    public DynamicObject createGrantBill(long user,DataSet personAwardDataSet) {

        DynamicObject data = BusinessDataServiceHelper.newDynamicObject(grantBill);
        data.set("yrpg_user",user);
        //通过插件获取当前切换选择的组织
        Long id = RequestContext.get().getOrgId();
        data.set("org", id);//组织赋值  重要
        data.set("yrpg_period", 120230050);//期间？
        data.set("billstatus", "A");//暂存

        DynamicObjectCollection entryEntity = data.getDynamicObjectCollection("yrpg_totalentryentity");//工资汇总

        // TODO 构建奖项金额
        DataSet awardDataSet = personAwardDataSet.copy().groupBy(new String []{"yrpg_awardtype"}).finish();
        DynamicObjectCollection awardCollection = ORM.create().toPlainDynamicObjectCollection(awardDataSet.copy());
        for(DynamicObject awardCol : awardCollection){
            DynamicObject dynamicObject = entryEntity.addNew();
            dynamicObject.set("yrpg_prize", awardCol.getLong("yrpg_awardtype"));
        }

        // TODO 构建奖项明细数据源
        DynamicObjectCollection sourceTryEntity = data.getDynamicObjectCollection("yrpg_sourceentryentity1");//工资明细
        DynamicObjectCollection personAwardCollection = ORM.create().toPlainDynamicObjectCollection(personAwardDataSet.copy());
        for(DynamicObject personAwardCol : personAwardCollection){
            DynamicObject sourceentryDynamicObject = sourceTryEntity.addNew();
            sourceentryDynamicObject.set("yrpg_awardtype", personAwardCol.getLong("yrpg_awardtype"));//奖项
            sourceentryDynamicObject.set("yrpg_fromperiod", personAwardCol.getLong("period"));//期间
            sourceentryDynamicObject.set("yrpg_beramount_in", personAwardCol.getBigDecimal("yrpg_beramount_in"));//收入
            sourceentryDynamicObject.set("yrpg_beramount_out", personAwardCol.getBigDecimal("yrpg_beramount_out"));//发出
            sourceentryDynamicObject.set("yrpg_awardamount", personAwardCol.getBigDecimal("yrpg_beramount_bal"));//余额
        }

        return data;


    }

    /**
     * 获取余额表数据
     */
    public void getDataSet(String index){

        //QFilter periodQf = new QFilter("period.periodyear", QCP.equals,"当前期间按年度");//期间-期间年度查询条件：
        if(index.equals("begin")) {
            //奖项余额表数据
            allAwardDataSet = QueryServiceHelper.queryDataSet(this.getClass().toString(), balanceBill,
                    "yrpg_user,period,yrpg_awardtype,yrpg_beramount_in,yrpg_beramount_out,yrpg_beramount_bal", null, null);
            //奖项类型数据
            awardTypeDataSet = QueryServiceHelper.queryDataSet(this.getClass().toString(), awardBill,
                    "id,yrpg_compute,yrpg_grand", null, null);
        }else if(index.equals("end")){
            //发放单
            grantAwardDataSet = QueryServiceHelper.queryDataSet(this.getClass().toString(), "yrpg_wages_grant",
                    "id,yrpg_user,yrpg_period,yrpg_totalentryentity.yrpg_prize as yrpg_awardtype", null, null);
        }
    }

    /**
     * 生成奖项明细
     */
    public void createAwardBill(){
        //按照期间查询所有人在该会计期间内的工资发放单 该奖项数据

        DynamicObjectCollection awardTypeCollection = ORM.create().toPlainDynamicObjectCollection(awardTypeDataSet.copy());
        for(DynamicObject awardTypeCol : awardTypeCollection){
            long awardId = awardTypeCol.getLong("id");//获取奖项
            String yrpgGrand = awardTypeCol.getString("yrpg_grand");//获取目标单标识

            //加载该奖项的的所有发放单
            List<Long> ids = new ArrayList<>();
            DataSet awardPersonDataSet = grantAwardDataSet.copy().filter("yrpg_awardtype = " + awardId);

            DynamicObjectCollection awardPersonCollection = ORM.create().toPlainDynamicObjectCollection(awardPersonDataSet.copy());
            for(DynamicObject awardPersonCol : awardPersonCollection){
                long id = awardPersonCol.getLong("id");

                ids.add(id);
            }


            //调用批量下推
            if(ids.size() > 0) {
                BillPushCommon.billPushAward(grantBill, yrpgGrand, ids);
            }
        }


    }




}